program serialt;

{
test framework for serial.pas

(c) Heiko Purnhagen 1995-1998
e-mail: purnhage@tnt.uni-hannover.de
WWW:    http://www.fet.uni-hannover.de/~purnhage/

This program may be copied according to the GNU General Public Licence.
See file COPYING for details.

980613 980614 980622 980811 980812
}


uses
  crt,serial,dos;

var
  w : word;
  b : byte;
  i : integer;
  hex : string[16];
  com : integer;
  baud : longint;
  usefifo : boolean;
  base : word;
  irq : integer;
  b1,b2 : boolean;
  c : char;


function b2b (b : byte) : string;
var
  i : integer;
  s : string;
begin
  s := '00000000';
  for i := 0 to 7 do
    if b and (1 shl i) <> 0 then
      s[8-i] := '1';
  b2b := s;
end;

function b2h (b : byte) : string;
begin
  b2h := hex[b div 16+1]+hex[b mod 16+1];
end;

function w2h (w : word) : string;
begin
  w2h := b2h(hi(w))+b2h(lo(w));
end;

procedure statusline;
var
  wx,wy : integer;
begin
  wx := wherex;
  wy := wherey;
  gotoxy(1,1);
  write ('t=',clkget/clkfreq:11:6,' rx=',serintrxbuf:4,' tx=',serinttxbuf:4,
         ' rxerr=',serrecerrstr(serrecerr),'   ');
  gotoxy(wx,wy);
end;

begin
  hex := '0123456789ABCDEF';
  writeln ('serial test   HP 980811');
  writeln ('serialt [<com> <baud> [<usefifo> [<base(dec)> <irq> (if com=5)]]]');
  writeln ('checkclk OK: ',clkcheck);
  for i := 1 to 4 do
  begin
    write ('COM',i,': ');
    w := serbase(i);
    write ('$',w2h(w),'=',w:4,' ');
    sertest (i,w,b1);
    if w > 0 then
    begin
      write (w2h(w));
      if b1 then
        writeln (' 16550 (fifo)')
      else
        writeln (' 8250 / 16450');
    end
    else
      writeln ('not available');
  end;

  if paramcount < 2 then
    exit;
  val(paramstr(1),com,i);
  if i <> 0 then
  begin
    writeln('value error: com');
    exit;
  end;
  val(paramstr(2),baud,i);
  if i <> 0 then
  begin
    writeln('value error: baud');
    exit;
  end;
  if paramcount < 3 then
    usefifo := true
  else
    usefifo := (paramstr(3) = 'y');
  if com = 5 then
  begin
    val(paramstr(4),base,i);
    if i <> 0 then
    begin
      writeln('value error: base');
      exit;
    end;
    val(paramstr(5),irq,i);
    if i <> 0 then
    begin
      writeln('value error: irq');
      exit;
    end;
    write ('COMx: ');
    w := base;
    write ('$',w2h(w),'=',w:4,' ');
    sertestbase (w,b1);
    if w > 0 then
    begin
      write (w2h(w));
      if b1 then
        writeln (' 16550 (fifo)')
      else
        writeln (' 8250 / 16450');
    end
    else
      writeln ('not available');
  end;

  writeln ('initing ...');
  b1 := serintinit(com,base,irq,baud,8,1,parityno,true,usefifo,b2);
  writeln ('serintinit=',b1,' usefifo=',usefifo,' fifodetect=',b2);
  if not b1 then
  begin
    write ('com loopback not successfull - start nevertheless (y/n) ?');
    readln (c);
    if not (c in ['y','Y']) then
      exit;
    b1 := serintinit(com,base,irq,baud,8,1,parityno,false,usefifo,b2);
    writeln ('serintinit=',b1,' usefifo=',usefifo,' fifodetect=',b2);
    if not b1 then
      exit;
  end;

  clkinit;
  clkreset;
  writeln ('^a stop  ^b tx-burst  ^c quit');
  write ('<ret>');
  readln;
  repeat
    statusline;
    c := ' ';
    if keypressed then
    begin
      c := readkey;
      if c=chr(1) then
        repeat
          statusline;
        until keypressed
      else if c=chr(2) then
        for i := 32 to 127 do
        begin
          serinttx(i);
          statusline;
        end
      else
        serinttx(ord(c));
      if sererror then
        writeln ('sertxerr');
    end;
    while serintrxbuf > 0 do
    begin
      write (chr(serintrx));
      if sererror then
        writeln ('serrxerr');
      statusline;
    end;
  until c=chr(3);

  writeln ('restoring ...');
  serintrestore;

  write ('<ret>');
  readln;

end.





var
  b1,b2,b3,b4 : boolean;
  chkrxto : boolean;

  exitsave : pointer;
  irqmask : byte;
  irqsave : pointer;
  intc,intbc : integer;
  intb : byte;

  regs : registers;

procedure irqhandler; interrupt;
begin
  port[$20] := $20;
  inc(intc);
  if port[$2f8+5] and 1 > 0 then
  begin
    intb := port[$2f8+0];
    inc(intbc);
  end;
end;

procedure myexit; far;
begin
  exitproc := exitsave;
  if irqsave <> NIL then
  begin
    setintvec($08+3,irqsave);
    port[$21] := irqmask;
  end;
end;

procedure initexit;
begin
  irqsave := NIL;
  exitsave := exitproc;
  exitproc := @myexit;
end;





  serselect (2);
  writeln ('serselect error: ',sererror);
  serinit (4800,8,1,parityno);
  writeln ('serinit error: ',sererror);
  sersetfifo (false,false);

  writeln ('02f8+4= ',b2b(port[$2f8+4]));
  writeln ('0021= ',b2b(port[$21]));
  writeln ('00a1= ',b2b(port[$a1]));

  { COM2 -> IRQ3 -> int8+3 }
  initexit;
  irqmask := port[$21];
  getintvec($08+3,irqsave);

  intc := 0;
  intb := 0;
  intbc := 0;

  setintvec($08+3,@irqhandler);
  port[$21] := irqmask and not $08;
  writeln ('0021= ',b2b(port[$21]));
  b := port[$2f8];
  port[$2f8+3] := port[$2f8+3] and $7f;
  port[$2f8+1] := 1;
  port[$2f8+4] := port[$2f8+4] or $08;
  port[$2f8+2] := $c1;

  intr($0b,regs);

  clkreset;
  repeat
    gotoxy(1,wherey);
    write (clkget:16,intc:8,intbc:8,' ',b2h(intb));
  until keypressed;

  port[$2f8+1] := 0;
  port[$2f8+4] := port[$2f8+4] and $f7;
  writeln;

  myexit;

  write ('<ret>');
  readln;
end.



  if paramcount <> 2 then
    exit;
  val(paramstr(1),com,i);
  val(paramstr(2),baud,i);
  serselect (com);
  writeln ('serselect error: ',sererror);
  serinit (baud,8,1,parityno);
  writeln ('serinit error: ',sererror);
  sersetfifo (true,true);
  write ('usetxcts?');
  readln (i);
  b1 := i<>0;
  write ('userxrts?');
  readln (i);
  b2 := i<>0;
  sersethandshake (b1,b2);
  sersettimeout (true,true,longtimeout);
  write ('dtr?');
  readln (i);
  b1 := i<>0;
  write ('rts?');
  readln (i);
  b2 := i<>0;
  write ('chkrxto?');
  readln (i);
  chkrxto := i<>0;
  sersetstatus (b1,b2);
  sergetstatus (b1,b2,b3,b4);
  writeln ('cts=',b1,' dsr=',b2,' ri=',b3,' dcd=',b4);
  writeln ('q to quit');
  repeat
    c := ' ';
    if keypressed then
    begin
      c := readkey;
      sertx(ord(c));
      if sertimeout then
        writeln ('txto');
    end;
    if chkrxto or serrxrdy(true) then
    begin
      write (chr(serrx));
      if sertimeout then
        writeln ('rxto');
    end;
    if serrecerr <> 0 then
    begin
      writeln (serrecerr);
      serrecerr := 0;
    end;
  until c='q';
  serrestore;
end.
